package pitapigui;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.select.SelectorComposer;
import org.zkoss.zk.ui.select.annotation.Listen;
import org.zkoss.zk.ui.select.annotation.Wire;
import org.zkoss.zul.Button;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Grid;
import org.zkoss.zul.Label;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Radiogroup;
import org.zkoss.zul.Row;
import org.zkoss.zul.Rows;
import org.zkoss.zul.Textbox;

public class Events extends SelectorComposer<Component> {

    private static final Logger log = Logger.getLogger(Events.class.getName());
    private final String baseURL = "http://smw-rda.esc.rzg.mpg.de/rdapit-0.1/pitapi/";

    @Wire
    private Button resolvePID;
    @Wire
    private Grid displayGrid;
    @Wire
    private Textbox pidInput;
    @Wire
    private Radiogroup typeOfPID;
    @Wire
    private Textbox typePID;
    @Wire
    private Textbox jsonText;
    @Wire
    private Checkbox showNames;

    @Listen("onClick = #peek")
    public void onPeek(Event ev) {
        String thePID = pidInput.getText().replace("/", "%2F");
        log.info("Input PID: " + thePID);
        Messagebox.show("The PID is a: " + peek(thePID));
    }

    @Listen("onClick = #validate")
    public void onValidate(Event ev) throws IOException {
        String thePID = pidInput.getText().replace("/", "%2F");
        log.info("Input PID: " + thePID);

        String validateType = typePID.getText();
        log.info("Type to validate: " + validateType);

        if (validate(thePID, validateType)) {
            Messagebox.show("Validation was successful!");
        } else {
            Messagebox.show("Validation was NOT successful!");
        }

    }

    private String peek(String pid) {
        String result = "";
        try {
            String urlToCall = baseURL + "peek/" + pid;
            log.info("URL to call: " + urlToCall);
            result = callWebservice(urlToCall);
        } catch (IOException ex) {
            Logger.getLogger(Events.class.getName()).log(Level.SEVERE, null, ex);
        }

        return result;

    }

    private boolean validate(String pid, String type) {
        boolean result = false;

        //http://smw-rda.esc.rzg.mpg.de/rdapit-0.1/pitapi/pid/11043.4/pitapi_test1?filter_by_type=11314.2%2Fffda2cfeb37a7ef8d87dd4b0723eb1f8
        // 11043.4/pitapi_test1
        //11314.2%2Fffda2cfeb37a7ef8d87dd4b0723eb1f8
        String urlToCall = baseURL + "pid/" + pid + "?filter_by_type=" + type;

        log.info("URL to call: " + urlToCall);

        try {
            String jsonCode = callWebservice(urlToCall);

            ObjectMapper mapper = new ObjectMapper();
            JsonNode node = mapper.readTree(jsonCode);

            JsonNode conformance = node.get("conformance");

            System.out.println("CONFORMANCE: " + conformance.toString());

            result = conformance.get(type.replace("%2F", "/")).asBoolean();

        } catch (IOException ex) {
            Logger.getLogger(Events.class.getName()).log(Level.SEVERE, null, ex);
        }

        return result;
    }

    @Listen("onClick = #resolvePID")
    public void onResolvePID(Event ev) {
        String thePID = pidInput.getText().replace("/", "%2F");
        log.info("Input PID: " + thePID);

        String selectedTypeOfPID = typeOfPID.getSelectedItem().getLabel();
        log.info("Selected type of PID: " + selectedTypeOfPID);

        String urlToCall = "";
        String jsonCode = "";
        String radioSelect = "";

        ArrayList<Row> rows = new ArrayList<Row>();

        if (selectedTypeOfPID.equals("Property")) {
            radioSelect = "property";
        } else if (selectedTypeOfPID.equals("Type")) {
            radioSelect = "type";
        } else if (selectedTypeOfPID.equals("Object")) {
            radioSelect = "pid";
            // --- To be done
        }

        urlToCall = baseURL + radioSelect + "/" + thePID;

        if (selectedTypeOfPID.equals("Object")) {
            urlToCall = urlToCall + "?include_property_names=true";
        }

        log.info("URL to call: " + urlToCall);

        try {
            jsonCode = callWebservice(urlToCall);
            log.info("Json Code: " + jsonCode);

            if (selectedTypeOfPID.equals("Object")) {
                rows = object2rows(jsonCode);
            } else if (selectedTypeOfPID.equals("Type")) {
                rows = type2rows(jsonCode);

            } else {
                rows = property2rows(jsonCode);
            }
            displayGrid.getRows().getChildren().clear();

            for (Row r : rows) {
                displayGrid.getRows().appendChild(r);
            }
        } catch (IOException ex) {
            Messagebox.show("Sorry, the PID doesn't seems to be a " + radioSelect);
        }

    }

    private ArrayList<Row> type2rows(String jsoncode) {
        ArrayList<Row> rows = new ArrayList<Row>();
        try {

            ObjectMapper mapper = new ObjectMapper();
            JsonNode node = mapper.readTree(jsoncode);

//            JsonNode optional = node.get("optionalProperties");
//            JsonNode mandatory = node.get("mandatoryProperties");
//
//            if (optional.isArray()) {
//                for (final JsonNode objNode : optional) {
//                    Row r = new Row();
//                    r.appendChild(new Label("Optional"));
//                    r.appendChild(new Label(objNode.textValue()));
//                    rows.add(r);
//                }
//            }
//
//            if (mandatory.isArray()) {
//                for (final JsonNode objNode : mandatory) {
//                    Row r = new Row();
//                    r.appendChild(new Label("Mandatory"));
//                    r.appendChild(new Label(objNode.textValue()));
//                    rows.add(r);
//                }
//            }
            Iterator<Entry<String, JsonNode>> nodeIterator = node.fields();

            while (nodeIterator.hasNext()) {

                Map.Entry<String, JsonNode> entry = (Map.Entry<String, JsonNode>) nodeIterator.next();
                //System.out.println("key --> " + entry.getKey() + " value-->" + entry.getValue());

                Row row = new Row();

                if (entry.getKey().equals("mandatoryProperties") | entry.getKey().equals("optionalProperties")) {
                    row.appendChild(new Label(entry.getKey()));
                    String temp = "";
                    for (final JsonNode objNode : entry.getValue()) {
                        if (showNames.isChecked()) {
                            temp = temp + getPropertyName(objNode.textValue()) + " ";
                        } else {
                            temp = temp + objNode.textValue() + " ";
                        }
                    }
                    temp = temp.trim().replace(" ", ", ");
                    row.appendChild(new Label(temp));
                } else {
                    row.appendChild(new Label(entry.getKey()));
                    row.appendChild(new Label(entry.getValue().asText()));
                }
                rows.add(row);
            }

        } catch (IOException ex) {
            Logger.getLogger(Events.class.getName()).log(Level.SEVERE, null, ex);
        }
        return rows;
    }

    private ArrayList<Row> object2rows(String jsoncode) {
        ArrayList<Row> rows = new ArrayList<Row>();

        try {

            ObjectMapper mapper = new ObjectMapper();
            JsonNode node = mapper.readTree(jsoncode);

            JsonNode values = node.get("values");

            Iterator<Entry<String, JsonNode>> nodeIterator = values.fields();

            while (nodeIterator.hasNext()) {

                Map.Entry<String, JsonNode> entry = (Map.Entry<String, JsonNode>) nodeIterator.next();
                //System.out.println("key --> " + entry.getKey() + " value-->" + entry.getValue());

                Row row = new Row();
                row.appendChild(new Label(entry.getValue().get("name").asText()));
                row.appendChild(new Label(entry.getValue().get("value").asText()));
                rows.add(row);

            }
        } catch (IOException ex) {
            Logger.getLogger(Events.class.getName()).log(Level.SEVERE, null, ex);
        }
        return rows;
    }

    private String getPropertyName(String pid) {
        String result = "";
        try {

            String urlToCall = baseURL + "property/" + pid;
            String jsonCode = callWebservice(urlToCall);

            ObjectMapper mapper = new ObjectMapper();
            JsonNode node = mapper.readTree(jsonCode);

            result = node.get("name").textValue();
        } catch (IOException ex) {
            Logger.getLogger(Events.class.getName()).log(Level.SEVERE, null, ex);
        }
        return result;

    }

    private ArrayList<Row> property2rows(String jsoncode) {
        ArrayList<Row> rows = new ArrayList<Row>();

        try {

            ObjectMapper mapper = new ObjectMapper();
            JsonNode node = mapper.readTree(jsoncode);

            Iterator<Entry<String, JsonNode>> nodeIterator = node.fields();

            while (nodeIterator.hasNext()) {

                Map.Entry<String, JsonNode> entry = (Map.Entry<String, JsonNode>) nodeIterator.next();
                //System.out.println("key --> " + entry.getKey() + " value-->" + entry.getValue());

                Row row = new Row();
                row.appendChild(new Label(entry.getKey()));
                row.appendChild(new Label(entry.getValue().asText()));
                rows.add(row);

            }
        } catch (IOException ex) {
            Logger.getLogger(Events.class.getName()).log(Level.SEVERE, null, ex);
        }
        return rows;
    }

    private String callWebservice(String urlToCall) throws IOException {
        StringBuilder sb = new StringBuilder("");

        try {
            URL url = new URL(urlToCall);
            BufferedReader in = new BufferedReader(
                    new InputStreamReader(url.openStream()));

            String inputLine;
            while ((inputLine = in.readLine()) != null) {
                sb.append(inputLine);
            }
            in.close();
        } catch (MalformedURLException ex) {
            Logger.getLogger(Events.class.getName()).log(Level.SEVERE, null, ex);
            jsonText.setText("An error occurred:\n-----------------\n" + ex.getMessage());
        }

        String temp = "URL: " + urlToCall + "\n--------\n" + sb.toString();

        jsonText.setText(temp);

        return sb.toString();
    }

}
